"""Version-pair mapping data modules.

Each module declares one :class:`~endstone_endweave.protocol.mapping_data.MappingData`
describing the ID changes between two adjacent protocol versions.
Both directions of a version pair import from the same module.
"""
